@@ -15,7 +15,7 @@ from ..core.util import (basestring, sanitize_identifier,
                          group_sanitizer, label_sanitizer, max_range,
                          find_range, dimension_sanitizer, OrderedDict,
                          bytes_to_unicode, unicode, dt64_to_dt, unique_array,
-                         builtins, config)
+                         builtins, config, dimension_range, disable_constant)
 from .options import Store, StoreOptions
 from .pprint import PrettyPrinter
 
@@ -41,6 +41,25 @@ def param_aliases(d):
     return d
 
 
+def process_dimensions(kdims, vdims):
+    """
+    Processes kdims and vdims specifications into a dictionary
+    of dimensions which can be passed to params.
+    """
+    dimensions = {}
+    for group, dims in [('kdims', kdims), ('vdims', vdims)]:
+        if dims is None:
+            continue
+        elif isinstance(dims, (tuple, basestring, Dimension)):
+            dims = [dims]
+        elif not isinstance(dims, list):
+            raise ValueError("%s must be a Dimension or list of dimensions, "
+                             "specified as tuples, string or Dimension instances, "
+                             "not %s." % (group, dims))
+        dimensions[group] = [d if isinstance(d, Dimension) else Dimension(d) for d in dims]
+    return dimensions
+
+
 class redim(object):
     """
     Utility that supports re-dimensioning any HoloViews object via the
@@ -136,13 +155,15 @@ class redim(object):
         if self.mode != 'dynamic':
             return redimmed.clone(kdims=kdims, vdims=vdims)
 
-        redimmed = redimmed.clone(kdims=kdims, vdims=vdims,
-                                  data= self._filter_cache(redimmed, kdims))
         from ..util import Dynamic
         def dynamic_redim(obj, **dynkwargs):
             return obj.redim(specs, **dimensions)
-        return Dynamic(redimmed, shared_data=True, streams=parent.streams,
-                       operation=dynamic_redim)
+        dmap = Dynamic(parent, streams=parent.streams, operation=dynamic_redim)
+        dmap.data = OrderedDict(self._filter_cache(redimmed, kdims))
+        with disable_constant(dmap):
+            dmap.kdims = kdims
+            dmap.vdims = vdims
+        return dmap
 
 
     def _redim(self, name, specs, **dims):
@@ -301,6 +322,9 @@ class Dimension(param.Parameterized):
 
         all_params = dict(existing_params, **params)
         if isinstance(spec, tuple):
+            if not all(isinstance(s, basestring) for s in spec) or len(spec) != 2:
+                raise ValueError("Dimensions specified as a tuple must be a tuple "
+                                 "consisting of the name and label not: %s" % str(spec)) 
             name, label = spec
             all_params['name'] = name
             all_params['label'] = label
@@ -783,35 +807,11 @@ class Dimensioned(LabelledData):
     _dim_aliases = dict(key_dimensions='kdims', value_dimensions='vdims',
                         constant_dimensions='cdims', deep_dimensions='ddims')
 
-
-    # Long-name aliases
-
-    @property
-    def key_dimensions(self): return self.kdims
-
-    @property
-    def value_dimensions(self): return self.vdims
-
-    @property
-    def constant_dimensions(self): return self.cdims
-
-    @property
-    def deep_dimensions(self): return self.ddims
-
-    def __init__(self, data, **params):
-        for group in self._dim_groups+list(self._dim_aliases.keys()):
-            if group in ['deep_dimensions', 'ddims']: continue
-            if group in params:
-                if group in self._dim_aliases:
-                    params[self._dim_aliases[group]] = params.pop(group)
-                    group = self._dim_aliases[group]
-                if group == 'cdims':
-                    dimensions = {d if isinstance(d, Dimension) else Dimension(d): val
-                                  for d, val in params.pop(group).items()}
-                else:
-                    dimensions = [d if isinstance(d, Dimension) else Dimension(d)
-                                  for d in params.pop(group)]
-                params[group] = dimensions
+    def __init__(self, data, kdims=None, vdims=None, **params):
+        params.update(process_dimensions(kdims, vdims))
+        if 'cdims' in params:
+            params['cdims'] = {d if isinstance(d, Dimension) else Dimension(d): val
+                               for d, val in params['cdims'].items()}
         super(Dimensioned, self).__init__(data, **params)
         self.ndims = len(self.kdims)
         cdims = [(d.name, val) for d, val in self.cdims.items()]
@@ -865,7 +865,7 @@ class Dimensioned(LabelledData):
                    'v': (lambda x: x.vdims, {}),
                    'c': (lambda x: x.cdims, {})}
         aliases = {'key': 'k', 'value': 'v', 'constant': 'c'}
-        if selection == 'all':
+        if selection in ['all', 'ranges']:
             groups = [d for d in self._dim_groups if d != 'cdims']
             dims = [dim for group in groups
                     for dim in getattr(self, group)]
@@ -1053,29 +1053,21 @@ class Dimensioned(LabelledData):
         dimension = self.get_dimension(dimension)
         if dimension is None:
             return (None, None)
-        if None not in dimension.range:
+        elif all(v is not None and np.isfinite(v) for v in dimension.range):
             return dimension.range
         elif data_range:
             if dimension in self.kdims+self.vdims:
                 dim_vals = self.dimension_values(dimension.name)
-                drange = find_range(dim_vals)
+                lower, upper = find_range(dim_vals)
             else:
                 dname = dimension.name
                 match_fn = lambda x: dname in x.kdims + x.vdims
                 range_fn = lambda x: x.range(dname)
                 ranges = self.traverse(range_fn, [match_fn])
-                drange = max_range(ranges)
-            soft_range = [r for r in dimension.soft_range if r is not None]
-            if soft_range:
-                drange = max_range([drange, soft_range])
+                lower, upper = max_range(ranges)
         else:
-            drange = dimension.soft_range
-        if dimension.range[0] is not None:
-            return (dimension.range[0], drange[1])
-        elif dimension.range[1] is not None:
-            return (drange[0], dimension.range[1])
-        else:
-            return drange
+            lower, upper = (np.NaN, np.NaN)
+        return dimension_range(lower, upper, dimension)
 
 
     def __repr__(self):
