@@ -8,7 +8,7 @@ from ..core.data import ImageInterface
 from ..core import Dimension, Element2D, Overlay, Dataset
 from ..core.boundingregion import BoundingRegion, BoundingBox
 from ..core.sheetcoords import SheetCoordinateSystem, Slice
-from ..core.util import max_range, dimension_range, compute_density, datetime_types
+from ..core.util import dimension_range, compute_density, datetime_types
 from .chart import Curve
 from .tabular import Table
 from .util import compute_edges, compute_slice_bounds, categorical_aggregate2d
@@ -230,7 +230,10 @@ class Image(Dataset, Raster, SheetCoordinateSystem):
     def __init__(self, data, kdims=None, vdims=None, bounds=None, extents=None,
                  xdensity=None, ydensity=None, **params):
         extents = extents if extents else (None, None, None, None)
-        if data is None: data = np.array([[0]])
+        if (data is None
+            or (isinstance(data, (list, tuple)) and not data)
+            or (isinstance(data, np.ndarray) and data.size == 0)):
+            data = np.zeros((2, 2))
         Dataset.__init__(self, data, kdims=kdims, vdims=vdims, extents=extents, **params)
 
         dim2, dim1 = self.interface.shape(self, gridded=True)[:2]
