@@ -1,6 +1,9 @@
 from __future__ import absolute_import
 
+from distutils.version import LooseVersion
+
 import numpy as np
+import bokeh
 from bokeh.palettes import all_palettes
 
 from ...core import (Store, Overlay, NdOverlay, Layout, AdjointLayout,
@@ -9,8 +12,15 @@ from ...element import (Curve, Points, Scatter, Image, Raster, Path,
                         RGB, Histogram, Spread, HeatMap, Contours, Bars,
                         Box, Bounds, Ellipse, Polygons, BoxWhisker, Arrow,
                         ErrorBars, Text, HLine, VLine, Spline, Spikes,
-                        Table, ItemTable, Area, HSV, QuadMesh, VectorField)
+                        Table, ItemTable, Area, HSV, QuadMesh, VectorField,
+                        Graph, Nodes, EdgePaths, Distribution, Bivariate)
 from ...core.options import Options, Cycle, Palette
+from ...core.util import VersionError
+
+if LooseVersion(bokeh.__version__) < '0.12.10':
+    raise VersionError("The bokeh extension requires a bokeh version >=0.12.10, "
+                       "please upgrade from bokeh %s to a more recent version."
+                       % bokeh.__version__, bokeh.__version__, '0.12.10')
 
 try:
     from ...interface import DFrame
@@ -23,12 +33,14 @@ from .element import OverlayPlot, ElementPlot
 from .chart import (PointPlot, CurvePlot, SpreadPlot, ErrorPlot, HistogramPlot,
                     SideHistogramPlot, BarPlot, SpikesPlot, SideSpikesPlot,
                     AreaPlot, VectorFieldPlot, BoxWhiskerPlot)
+from .graphs import GraphPlot, NodePlot
 from .path import PathPlot, PolygonPlot, ContourPlot
 from .plot import GridPlot, LayoutPlot, AdjointLayoutPlot
 from .raster import RasterPlot, RGBPlot, HeatMapPlot, HSVPlot, QuadMeshPlot
 from .renderer import BokehRenderer
+from .stats import DistributionPlot, BivariatePlot
 from .tabular import TablePlot
-from .util import bokeh_version
+from .util import bokeh_version # noqa (API import)
 
 
 Store.renderers['bokeh'] = BokehRenderer.instance()
@@ -81,9 +93,18 @@ associations = {Overlay: OverlayPlot,
                 Spline: SplinePlot,
                 Arrow: ArrowPlot,
 
+                # Graph Elements
+                Graph: GraphPlot,
+                Nodes: NodePlot,
+                EdgePaths: PathPlot,
+
                 # Tabular
                 Table: TablePlot,
-                ItemTable: TablePlot}
+                ItemTable: TablePlot,
+
+                # Statistics
+                Distribution: DistributionPlot,
+                Bivariate: BivariatePlot}
 
 if DFrame is not None:
     associations[DFrame] = TablePlot
@@ -102,8 +123,6 @@ else:
         framedcls.show_frame = True
 
 
-
-
 AdjointLayoutPlot.registry[Histogram] = SideHistogramPlot
 AdjointLayoutPlot.registry[Spikes] = SideSpikesPlot
 
@@ -140,18 +159,19 @@ options.Spread = Options('style', color=Cycle(), alpha=0.6, line_color='black')
 options.Bars = Options('style', color=Cycle(), line_color='black', width=0.8)
 
 options.Spikes = Options('style', color='black', cmap='fire')
-options.Area = Options('style', color=Cycle(), line_color='black')
+options.Area = Options('style', color=Cycle(), alpha=1, line_color='black')
 options.VectorField = Options('style', color='black')
 
 # Paths
-options.Contours = Options('style', color=Cycle())
 if not config.style_17:
     options.Contours = Options('plot', show_legend=True)
-options.Path = Options('style', color=Cycle())
+options.Contours = Options('style', color=Cycle(), cmap='viridis')
+options.Path = Options('style', color=Cycle(), cmap='viridis')
 options.Box = Options('style', color='black')
 options.Bounds = Options('style', color='black')
 options.Ellipse = Options('style', color='black')
-options.Polygons = Options('style', color=Cycle(), line_color='black')
+options.Polygons = Options('style', color=Cycle(), line_color='black',
+                           cmap='viridis')
 
 # Rasters
 options.Image = Options('style', cmap=dflt_cmap)
@@ -165,16 +185,41 @@ options.HLine = Options('style', color=Cycle(), line_width=3, alpha=1)
 options.VLine = Options('style', color=Cycle(), line_width=3, alpha=1)
 options.Arrow = Options('style', arrow_size=10)
 
+# Graphs
+options.Graph = Options('style', node_size=20, node_fill_color=Cycle(),
+                        node_line_color='black',
+                        node_selection_fill_color='limegreen',
+                        node_nonselection_fill_color=Cycle(),
+                        node_hover_line_color='black',
+                        node_hover_fill_color='indianred',
+                        node_nonselection_alpha=0.2,
+                        edge_nonselection_alpha=0.2,
+                        edge_line_color='black', edge_line_width=2,
+                        edge_nonselection_line_color='black',
+                        edge_hover_line_color='indianred',
+                        edge_selection_line_color='limegreen')
+options.Nodes = Options('style', line_color='black', color=Cycle(),
+                        size=20, nonselection_fill_color=Cycle(),
+                        selection_fill_color='limegreen',
+                        hover_fill_color='indianred')
+options.Nodes = Options('plot', tools=['hover', 'tap'])
+options.EdgePaths = Options('style', color='black', nonselection_alpha=0.2,
+                            line_width=2, selection_color='limegreen',
+                            hover_line_color='indianred')
+options.EdgePaths = Options('plot', tools=['hover', 'tap'])
+
 # Define composite defaults
 options.GridMatrix = Options('plot', shared_xaxis=True, shared_yaxis=True,
                              xaxis=None, yaxis=None)
 
-if bokeh_version >= '0.12.5':
-    options.Overlay = Options('style', click_policy='mute')
-    options.NdOverlay = Options('style', click_policy='mute')
-    options.Curve = Options('style', muted_alpha=0.2)
-    options.Path = Options('style', muted_alpha=0.2)
-    options.Scatter = Options('style', muted_alpha=0.2)
-    options.Points = Options('style', muted_alpha=0.2)
-    options.Polygons = Options('style', muted_alpha=0.2)
+options.Overlay = Options('style', click_policy='mute')
+options.NdOverlay = Options('style', click_policy='mute')
+options.Curve = Options('style', muted_alpha=0.2)
+options.Path = Options('style', muted_alpha=0.2)
+options.Scatter = Options('style', muted_alpha=0.2)
+options.Points = Options('style', muted_alpha=0.2)
+options.Polygons = Options('style', muted_alpha=0.2)
 
+# Statistics
+options.Distribution = Options('style', fill_color=Cycle(), line_color='black',
+                               fill_alpha=0.5)
