@@ -1,15 +1,15 @@
 from collections import defaultdict
-from bokeh.models import CustomJS
+
+from bokeh.models import CustomJS, FactorRange, DatetimeAxis
 
 from ...core import OrderedDict
 from ...streams import (Stream, PointerXY, RangeXY, Selection1D, RangeX,
-                        RangeY, PointerX, PointerY, Bounds, BoundsX, BoundsY,
+                        RangeY, PointerX, PointerY, BoundsX, BoundsY,
                         Tap, SingleTap, DoubleTap, MouseEnter, MouseLeave,
-                        PlotSize, Draw)
-from ...streams import PositionX, PositionY, PositionXY # Deprecated: remove in 2.0
-from ..comms import JupyterCommJS
-from .util import bokeh_version
-
+                        PlotSize, Draw, BoundsXY, PlotReset)
+from ...streams import PositionX, PositionY, PositionXY, Bounds # Deprecated: remove in 2.0
+from ..comms import JupyterCommJS, Comm
+from .util import convert_timestamp
 
 
 class MessageCallback(object):
@@ -51,7 +51,10 @@ class MessageCallback(object):
         self.plot = plot
         self.streams = streams
         if plot.renderer.mode != 'server':
-            self.comm = self._comm_type(plot, on_msg=self.on_msg)
+            try:
+                self.comm = self._comm_type(plot, on_msg=self.on_msg)
+            except AttributeError:
+                self.comm = Comm(plot)
         self.source = source
         self.handle_ids = defaultdict(dict)
         self.callbacks = []
@@ -151,7 +154,7 @@ class CustomJSCallback(MessageCallback):
             var unique_events = [];
             for (var i=0; i<events.length; i++) {{
                 [event, data] = events[i];
-                if (!unique_events.includes(event)) {{
+                if (unique_events.indexOf(event)===-1) {{
                     unique.unshift(data);
                     unique_events.push(event);
                 }}
@@ -307,7 +310,7 @@ class CustomJSCallback(MessageCallback):
         code and gathering all plotting handles and installs it on
         the requested callback handle.
         """
-        if self.on_events and bokeh_version >= '0.12.5':
+        if self.on_events:
             for event in self.on_events:
                 handle.js_on_event(event, js_callback)
         elif self.on_changes:
@@ -326,6 +329,11 @@ class ServerCallback(MessageCallback):
     Stream(s) attached to the callback.
     """
 
+    def __init__(self, plot, streams, source, **params):
+        super(ServerCallback, self).__init__(plot, streams, source, **params)
+        self._active = False
+
+
     @classmethod
     def resolve_attr_spec(cls, spec, cb_obj, model=None):
         """
@@ -356,8 +364,9 @@ class ServerCallback(MessageCallback):
         value change at once rather than firing off multiple plot updates.
         """
         self._queue.append((attr, old, new))
-        if self.process_on_change not in self.plot.document._session_callbacks:
+        if not self._active:
             self.plot.document.add_timeout_callback(self.process_on_change, 50)
+            self._active = True
 
 
     def on_event(self, event):
@@ -366,16 +375,18 @@ class ServerCallback(MessageCallback):
         value change at once rather than firing off multiple plot updates.
         """
         self._queue.append((event))
-        if self.process_on_event not in self.plot.document._session_callbacks:
+        if not self._active:
             self.plot.document.add_timeout_callback(self.process_on_event, 50)
-
+            self._active = True
 
     def process_on_event(self):
         """
         Trigger callback change event and triggering corresponding streams.
         """
         if not self._queue:
+            self._active = False
             return
+        self._queue = []
         # Get unique event types in the queue
         events = list(OrderedDict([(event.event_name, event)
                                    for event in self._queue]).values())
@@ -393,6 +404,7 @@ class ServerCallback(MessageCallback):
 
     def process_on_change(self):
         if not self._queue:
+            self._active = False
             return
         self._queue = []
 
@@ -415,7 +427,7 @@ class ServerCallback(MessageCallback):
         """
         Set up on_change events for bokeh server interactions.
         """
-        if self.on_events and bokeh_version >= '0.12.5':
+        if self.on_events:
             for event in self.on_events:
                 handle.on_event(event, self.on_event)
         elif self.on_changes:
@@ -537,6 +549,22 @@ class PointerXYCallback(Callback):
                data['y'] = y_range.end }}
            """
 
+    def _process_msg(self, msg):
+        x_range = self.plot.handles.get('x_range')
+        y_range = self.plot.handles.get('y_range')
+        if isinstance(x_range, FactorRange) and isinstance(msg.get('x'), (int, float)):
+            msg['x'] = x_range.factors[int(msg['x'])]
+        if isinstance(y_range, FactorRange) and isinstance(msg.get('y'), (int, float)):
+            msg['y'] = y_range.factors[int(msg['y'])]
+
+        xaxis = self.plot.handles.get('xaxis')
+        yaxis = self.plot.handles.get('yaxis')
+        if 'x' in msg and isinstance(xaxis, DatetimeAxis):
+            msg['x'] = convert_timestamp(msg['x'])
+        if 'y' in msg and isinstance(yaxis, DatetimeAxis):
+            msg['y'] = convert_timestamp(msg['y'])
+        return msg
+
 
 class PointerXCallback(PointerXYCallback):
     """
@@ -646,9 +674,21 @@ class RangeXYCallback(Callback):
     def _process_msg(self, msg):
         data = {}
         if 'x0' in msg and 'x1' in msg:
-            data['x_range'] = (msg['x0'], msg['x1'])
+            x0, x1 = msg['x0'], msg['x1']
+            if isinstance(self.plot.handles.get('xaxis'), DatetimeAxis):
+                x0 = convert_timestamp(x0)
+                x1 = convert_timestamp(x1)
+            if self.plot.invert_xaxis:
+                x0, x1 = x1, x0
+            data['x_range'] = (x0, x1)
         if 'y0' in msg and 'y1' in msg:
-            data['y_range'] = (msg['y0'], msg['y1'])
+            y0, y1 = msg['y0'], msg['y1']
+            if isinstance(self.plot.handles.get('yaxis'), DatetimeAxis):
+                y0 = convert_timestamp(y0)
+                y1 = convert_timestamp(y1)
+            if self.plot.invert_yaxis:
+                y0, y1 = y1, y0
+            data['y_range'] = (y0, y1)
         return data
 
 
@@ -661,12 +701,6 @@ class RangeXCallback(RangeXYCallback):
                   'x1': 'x_range.attributes.end'}
     models = ['x_range']
 
-    def _process_msg(self, msg):
-        if 'x0' in msg and 'x1' in msg:
-            return {'x_range': (msg['x0'], msg['x1'])}
-        else:
-            return {}
-
 
 class RangeYCallback(RangeXYCallback):
     """
@@ -677,11 +711,6 @@ class RangeYCallback(RangeXYCallback):
                   'y1': 'y_range.attributes.end'}
     models = ['y_range']
 
-    def _process_msg(self, msg):
-        if 'y0' in msg and 'y1' in msg:
-            return {'y_range': (msg['y0'], msg['y1'])}
-        else:
-            return {}
 
 
 class PlotSizeCallback(Callback):
@@ -713,10 +742,16 @@ class BoundsCallback(Callback):
     models = ['plot']
     extra_models = ['box_select']
     on_events = ['selectiongeometry']
-    skip = ["!box_select || !box_select.active"]
+    skip = ["cb_obj.geometry.type != 'rect'"]
 
     def _process_msg(self, msg):
         if all(c in msg for c in ['x0', 'y0', 'x1', 'y1']):
+            if isinstance(self.plot.handles.get('xaxis'), DatetimeAxis):
+                msg['x0'] = convert_timestamp(msg['x0'])
+                msg['x1'] = convert_timestamp(msg['x1'])
+            if isinstance(self.plot.handles.get('yaxis'), DatetimeAxis):
+                msg['y0'] = convert_timestamp(msg['y0'])
+                msg['y1'] = convert_timestamp(msg['y1'])
             return {'bounds': (msg['x0'], msg['y0'], msg['x1'], msg['y1'])}
         else:
             return {}
@@ -731,10 +766,13 @@ class BoundsXCallback(Callback):
     models = ['plot']
     extra_models = ['xbox_select']
     on_events = ['selectiongeometry']
-    skip = ["!xbox_select || !xbox_select.active"]
+    skip = ["cb_obj.geometry.type != 'rect'"]
 
     def _process_msg(self, msg):
         if all(c in msg for c in ['x0', 'x1']):
+            if isinstance(self.plot.handles.get('xaxis'), DatetimeAxis):
+                msg['x0'] = convert_timestamp(msg['x0'])
+                msg['x1'] = convert_timestamp(msg['x1'])
             return {'boundsx': (msg['x0'], msg['x1'])}
         else:
             return {}
@@ -749,10 +787,13 @@ class BoundsYCallback(Callback):
     models = ['plot']
     extra_models = ['ybox_select']
     on_events = ['selectiongeometry']
-    skip = ["!ybox_select || !ybox_select.active"]
+    skip = ["cb_obj.geometry.type != 'rect'"]
 
     def _process_msg(self, msg):
         if all(c in msg for c in ['y0', 'y1']):
+            if isinstance(self.plot.handles.get('yaxis'), DatetimeAxis):
+                msg['y0'] = convert_timestamp(msg['y0'])
+                msg['y1'] = convert_timestamp(msg['y1'])
             return {'boundsy': (msg['y0'], msg['y1'])}
         else:
             return {}
@@ -774,6 +815,18 @@ class Selection1DCallback(Callback):
             return {}
 
 
+class ResetCallback(Callback):
+    """
+    Signals the Reset stream if an event has been triggered.
+    """
+
+    models = ['plot']
+    on_events = ['reset']
+
+    def _process_msg(self, msg):
+        return {'reset': True}
+
+
 callbacks = Stream._callbacks['bokeh']
 
 callbacks[PointerXY]   = PointerXYCallback
@@ -788,11 +841,13 @@ callbacks[RangeXY]     = RangeXYCallback
 callbacks[RangeX]      = RangeXCallback
 callbacks[RangeY]      = RangeYCallback
 callbacks[Bounds]      = BoundsCallback
+callbacks[BoundsXY]    = BoundsCallback
 callbacks[BoundsX]     = BoundsXCallback
 callbacks[BoundsY]     = BoundsYCallback
 callbacks[Selection1D] = Selection1DCallback
 callbacks[PlotSize]    = PlotSizeCallback
-callbacks[Draw] = DrawCallback
+callbacks[Draw]        = DrawCallback
+callbacks[PlotReset]   = ResetCallback
 
 # Aliases for deprecated streams
 callbacks[PositionXY]  = PointerXYCallback
