@@ -7,7 +7,8 @@ import bokeh
 import bokeh.plotting
 from bokeh import palettes
 from bokeh.core.properties import value
-from bokeh.models import  HoverTool, Renderer, Range1d, DataRange1d, FactorRange
+from bokeh.models import (HoverTool, Renderer, Range1d, DataRange1d, Title,
+                          FactorRange, FuncTickFormatter, Tool, Legend)
 from bokeh.models.tickers import Ticker, BasicTicker, FixedTicker, LogTicker
 from bokeh.models.widgets import Panel, Tabs
 from bokeh.models.mappers import LinearColorMapper
@@ -18,20 +19,15 @@ except ImportError:
     LogColorMapper, ColorBar = None, None
 from bokeh.plotting.helpers import _known_tools as known_tools
 
-from ...core import Store, DynamicMap, CompositeOverlay, Element, Dimension
+from ...core import DynamicMap, CompositeOverlay, Element, Dimension
 from ...core.options import abbreviated_exception, SkipRendering
 from ...core import util
-from ...streams import Stream
+from ...streams import Stream, Buffer
 from ..plot import GenericElementPlot, GenericOverlayPlot
 from ..util import dynamic_update
 from .plot import BokehPlot, TOOLS
-from .util import (mpl_to_bokeh, get_tab_title, bokeh_version,
-                   mplcmap_to_palette, py2js_tickformatter, rgba_tuple)
-
-if bokeh_version >= '0.12':
-    from bokeh.models import FuncTickFormatter
-else:
-    FuncTickFormatter = None
+from .util import (mpl_to_bokeh, get_tab_title, mplcmap_to_palette,
+                   py2js_tickformatter, rgba_tuple, recursive_model_update)
 
 property_prefixes = ['selection', 'nonselection', 'muted', 'hover']
 
@@ -162,16 +158,15 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         tick locations or bokeh Ticker object. If set to None
         default bokeh ticking behavior is applied.""")
 
-    # The plot objects to be updated on each frame
-    # Any entries should be existing keys in the handles
-    # instance attribute.
-    _update_handles = ['source', 'glyph', 'glyph_renderer']
     _categorical = False
 
     # Declares the default types for continuous x- and y-axes
     _x_range_type = Range1d
     _y_range_type = Range1d
 
+    # Whether the plot supports streaming data
+    _stream_data = True
+
     def __init__(self, element, plot=None, **params):
         self.current_ranges = None
         super(ElementPlot, self).__init__(element, **params)
@@ -179,6 +174,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         self.static = len(self.hmap) == 1 and len(self.keys) == len(self.hmap)
         self.callbacks = self._construct_callbacks()
         self.static_source = False
+        self.streaming = [s for s in self.streams if isinstance(s, Buffer)]
 
         # Whether axes are shared between plots
         self._shared = {'x': False, 'y': False}
@@ -233,6 +229,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         tooltips, hover_opts = self._hover_opts(element)
         tooltips = [(ttp.pprint_label, '@{%s}' % util.dimension_sanitizer(ttp.name))
                     if isinstance(ttp, Dimension) else ttp for ttp in tooltips]
+        if not tooltips: tooltips = None
 
         callbacks = callbacks+self.callbacks
         cb_tools, tool_names = [], []
@@ -251,23 +248,31 @@ class ElementPlot(BokehPlot, GenericElementPlot):
 
         tools = [t for t in cb_tools + self.default_tools + self.tools
                  if t not in tool_names]
-        hover_tools = [t for t in tools if isinstance(t, HoverTool)]
-        if 'hover' in tools:
+
+        copied_tools = []
+        for tool in tools:
+            if isinstance(tool, Tool):
+                properties = tool.properties_with_values(include_defaults=False)
+                tool = type(tool)(**properties)
+            copied_tools.append(tool)
+
+        hover_tools = [t for t in copied_tools if isinstance(t, HoverTool)]
+        if 'hover' in copied_tools:
             hover = HoverTool(tooltips=tooltips, **hover_opts)
-            tools[tools.index('hover')] = hover
+            copied_tools[copied_tools.index('hover')] = hover
         elif any(hover_tools):
             hover = hover_tools[0]
         if hover:
             self.handles['hover'] = hover
-        return tools
+        return copied_tools
 
 
-    def _get_hover_data(self, data, element, empty=False):
+    def _get_hover_data(self, data, element):
         """
         Initializes hover data based on Element dimension values.
         If empty initializes with no data.
         """
-        if not any(isinstance(t, HoverTool) for t in self.state.tools):
+        if not any(isinstance(t, HoverTool) for t in self.state.tools) or self.static_source:
             return
 
         for d in element.dimensions():
@@ -317,7 +322,8 @@ class ElementPlot(BokehPlot, GenericElementPlot):
 
         plot_ranges = {}
         # Try finding shared ranges in other plots in the same Layout
-        if plots and self.shared_axes:
+        norm_opts = self.lookup_options(el, 'norm').options
+        if plots and self.shared_axes and not norm_opts.get('axiswise', False):
             plot_ranges = self._merge_ranges(plots, xlabel, ylabel)
 
         if el.get_dimension_type(0) in util.datetime_types:
@@ -392,10 +398,8 @@ class ElementPlot(BokehPlot, GenericElementPlot):
             properties['tools'] = tools
         properties['toolbar_location'] = self.toolbar
 
-        if bokeh_version < '0.12.6':
-            properties['webgl'] = self.renderer.webgl
-        elif self.renderer.webgl:
-            properties['output_backend'] =  'webgl'
+        if self.renderer.webgl:
+            properties['output_backend'] = 'webgl'
 
         with warnings.catch_warnings():
             # Bokeh raises warnings about duplicate tools but these
@@ -414,8 +418,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         plot_props = dict(plot_height=int(self.height*size_multiplier),
                           plot_width=int(self.width*size_multiplier),
                           sizing_mode=self.sizing_mode)
-        if bokeh_version < '0.12':
-            plot_props.update(self._title_properties(key, plot, element))
         if self.bgcolor:
             plot_props['background_fill_color'] = self.bgcolor
         if self.border is not None:
@@ -433,15 +435,11 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         else:
             title = ''
 
-        if bokeh_version < '0.12':
-            title_font = self._fontsize('title', 'title_text_font_size')
-            return dict(title=title, title_text_color='black', **title_font)
-        else:
-            opts = dict(text=title, text_color='black')
-            title_font = self._fontsize('title').get('fontsize')
-            if title_font:
-                opts['text_font_size'] = value(title_font)
-            return opts
+        opts = dict(text=title, text_color='black')
+        title_font = self._fontsize('title').get('fontsize')
+        if title_font:
+            opts['text_font_size'] = value(title_font)
+        return opts
 
     def _init_axes(self, plot):
         if self.xaxis is None:
@@ -497,11 +495,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                     labels = [l if isinstance(l, util.basestring) else str(l)
                               for l in labels]
                     axis_props['ticker'] = FixedTicker(ticks=ticks)
-                    if bokeh_version > '0.12.5':
-                        axis_props['major_label_overrides'] = dict(zip(ticks, labels))
-                    else:
-                        self.warning('Explicit tick labels not supported until'
-                                     'bokeh 0.12.6, please upgrade')
+                    axis_props['major_label_overrides'] = dict(zip(ticks, labels))
                 else:
                     axis_props['ticker'] = FixedTicker(ticks=ticker)
 
@@ -530,14 +524,21 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         if not len(dimensions) >= 2:
             dimensions = dimensions+[None]
         plot.update(**self._plot_properties(key, plot, element))
+
         props = {axis: self._axis_properties(axis, key, plot, dim)
                  for axis, dim in zip(['x', 'y'], dimensions)}
-        plot.xaxis[0].update(**props.get('x', {}))
-        plot.yaxis[0].update(**props.get('y', {}))
-
-        if bokeh_version >= '0.12' and not self.overlaid:
-            plot.title.update(**self._title_properties(key, plot, element))
+        xlabel, ylabel, zlabel = self._get_axis_labels(dimensions)
+        if self.invert_axes: xlabel, ylabel = ylabel, xlabel
+        props['x']['axis_label'] = xlabel
+        props['y']['axis_label'] = ylabel
+        recursive_model_update(plot.xaxis[0], props.get('x', {}))
+        recursive_model_update(plot.yaxis[0], props.get('y', {}))
 
+        if not self.overlaid:
+            if plot.title:
+                plot.title.update(**self._title_properties(key, plot, element))
+            else:
+                plot.title = Title(**self._title_properties(key, plot, element))
         if not self.show_grid:
             plot.xgrid.grid_line_color = None
             plot.ygrid.grid_line_color = None
@@ -557,33 +558,49 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         if any(isinstance(ax_range, FactorRange) for ax_range in [x_range, y_range]):
             xfactors, yfactors = self._get_factors(element)
         framewise = self.framewise
-        if not self.drawn or (not self.model_changed(x_range) and framewise):
-            self._update_range(x_range, l, r, xfactors, self.invert_xaxis, self._shared['x'], self.logx)
-        if not self.drawn or (not self.model_changed(y_range) and framewise):
-            self._update_range(y_range, b, t, yfactors, self.invert_yaxis, self._shared['y'], self.logy)
+        streaming = (self.streaming and any(stream._triggering for stream in self.streaming))
+        xupdate = ((not self.model_changed(x_range) and (framewise or streaming))
+                   or xfactors is not None)
+        yupdate = ((not self.model_changed(y_range) and (framewise or streaming))
+                   or yfactors is not None)
+        if not self.drawn or xupdate:
+            self._update_range(x_range, l, r, xfactors, self.invert_xaxis,
+                               self._shared['x'], self.logx, streaming)
+        if not self.drawn or yupdate:
+            self._update_range(y_range, b, t, yfactors, self.invert_yaxis,
+                               self._shared['y'], self.logy, streaming)
 
 
-    def _update_range(self, axis_range, low, high, factors, invert, shared, log):
+    def _update_range(self, axis_range, low, high, factors, invert, shared, log, streaming=False):
         if isinstance(axis_range, (Range1d, DataRange1d)) and self.apply_ranges:
-            if (low == high and low is not None and
-                not isinstance(high, util.datetime_types)):
-                offset = abs(low*0.1 if low else 0.5)
-                low -= offset
-                high += offset
+            if (low == high and low is not None):
+                if isinstance(low, util.datetime_types):
+                    offset = np.timedelta64(500, 'ms')
+                    low -= offset
+                    high += offset
+                else:
+                    offset = abs(low*0.1 if low else 0.5)
+                    low -= offset
+                    high += offset
             if invert: low, high = high, low
             if shared:
                 shared = (axis_range.start, axis_range.end)
                 low, high = util.max_range([(low, high), shared])
-            if log and low <= 0:
-                low = 0.01 if high < 0.01 else 10**(np.log10(high)-1)
+            if log and (low is None or low <= 0):
+                low = 0.01 if high < 0.01 else 10**(np.log10(high)-2)
                 self.warning("Logarithmic axis range encountered value less than or equal to zero, "
                              "please supply explicit lower-bound to override default of %.3f." % low)
+            updates = {}
             if low is not None and (isinstance(low, util.datetime_types)
                                     or np.isfinite(low)):
-                axis_range.start = low
+                updates['start'] = (axis_range.start, low)
             if high is not None and (isinstance(high, util.datetime_types)
                                      or np.isfinite(high)):
-                axis_range.end = high
+                updates['end'] = (axis_range.end, high)
+            for k, (old, new) in updates.items():
+                axis_range.update(**{k:new})
+                if streaming:
+                    axis_range.trigger(k, old, new)
         elif isinstance(axis_range, FactorRange):
             factors = list(factors)
             if invert: factors = factors[::-1]
@@ -616,8 +633,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                         for i in range(2)]
         coords = tuple([v if vals.dtype.kind in 'SU' else dim.pprint_value(v) for v in vals]
                   for dim, vals in [(xdim, xvals), (ydim, yvals)])
-        if bokeh_version < '0.12.7':
-            coords = tuple([v.replace(':', ';') for v in vals] for vals in coords)
         if self.invert_axes: coords = coords[::-1]
         return coords
 
@@ -627,10 +642,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         Disables legends if show_legend is disabled.
         """
         for l in self.handles['plot'].legend:
-            if bokeh_version > '0.12.2':
-                l.items[:] = []
-            else:
-                l.legends[:] = []
+            l.items[:] = []
             l.border_line_alpha = 0
             l.background_fill_alpha = 0
 
@@ -648,17 +660,15 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         return renderer, renderer.glyph
 
 
-    def _glyph_properties(self, plot, element, source, ranges):
-        properties = self.style[self.cyclic_index]
-
+    def _glyph_properties(self, plot, element, source, ranges, style):
+        properties = dict(style, source=source)
         if self.show_legend:
             if self.overlay_dims:
                 legend = ', '.join([d.pprint_value(v) for d, v in
                                     self.overlay_dims.items()])
             else:
                 legend = element.label
-            properties['legend'] = legend
-        properties['source'] = source
+            properties['legend'] = value(legend)
         return properties
 
     def _update_glyph(self, renderer, properties, mapping, glyph):
@@ -724,23 +734,22 @@ class ElementPlot(BokehPlot, GenericElementPlot):
 
 
     def _init_glyphs(self, plot, element, ranges, source):
-        empty = False
         style_element = element.last if self.batched else element
 
         # Get data and initialize data source
-        empty = False
         if self.batched:
             current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
-            data, mapping = self.get_batched_data(element, ranges, empty)
+            data, mapping, style = self.get_batched_data(element, ranges)
         else:
-            data, mapping = self.get_data(element, ranges, empty)
+            style = self.style[self.cyclic_index]
+            data, mapping, style = self.get_data(element, ranges, style)
             current_id = element._plot_id
         if source is None:
             source = self._init_datasource(data)
         self.handles['previous_id'] = current_id
         self.handles['source'] = source
 
-        properties = self._glyph_properties(plot, style_element, source, ranges)
+        properties = self._glyph_properties(plot, style_element, source, ranges, style)
         with abbreviated_exception():
             renderer, glyph = self._init_glyph(plot, mapping, properties)
         self.handles['glyph'] = glyph
@@ -803,7 +812,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         plot = self.handles['plot']
         glyph = self.handles.get('glyph')
         source = self.handles['source']
-        empty = False
         mapping = {}
 
         # Cache frame object id to skip updating data if unchanged
@@ -814,22 +822,23 @@ class ElementPlot(BokehPlot, GenericElementPlot):
             current_id = element._plot_id
         self.handles['previous_id'] = current_id
         self.static_source = (self.dynamic and (current_id == previous_id))
+        style = self.style[self.cyclic_index]
         if self.batched:
-            data, mapping = self.get_batched_data(element, ranges, empty)
+            data, mapping, style = self.get_batched_data(element, ranges)
         else:
-            data, mapping = self.get_data(element, ranges, empty)
+            data, mapping, style = self.get_data(element, ranges, style)
 
         if not self.static_source:
             self._update_datasource(source, data)
 
         if glyph:
-            properties = self._glyph_properties(plot, element, source, ranges)
+            properties = self._glyph_properties(plot, element, source, ranges, style)
             renderer = self.handles.get('glyph_renderer')
             with abbreviated_exception():
                 self._update_glyph(renderer, properties, mapping, glyph)
 
 
-    def update_frame(self, key, ranges=None, plot=None, element=None, empty=False):
+    def update_frame(self, key, ranges=None, plot=None, element=None):
         """
         Updates an existing plot with data corresponding
         to the key.
@@ -842,10 +851,15 @@ class ElementPlot(BokehPlot, GenericElementPlot):
             self.current_frame = element
 
         renderer = self.handles.get('glyph_renderer', None)
+        glyph = self.handles.get('glyph', None)
+        visible = bool(element)
         if hasattr(renderer, 'visible'):
-            renderer.visible = bool(element)
+            renderer.visible = visible
+        if hasattr(glyph, 'visible'):
+            glyph.visible = visible
 
-        if (self.batched and not element) or element is None or (not self.dynamic and self.static):
+        if ((self.batched and not element) or element is None or (not self.dynamic and self.static) or
+            (self.streaming and self.streaming[0].data is self.current_frame.data and not self.streaming[0]._triggering)):
             return
 
         if self.batched:
@@ -861,13 +875,12 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         self.set_param(**self.lookup_options(style_element, 'plot').options)
         ranges = util.match_spec(style_element, ranges)
         self.current_ranges = ranges
-        self._update_glyphs(element, ranges)
-
         plot = self.handles['plot']
         if not self.overlaid:
             self._update_ranges(style_element, ranges)
             self._update_plot(key, plot, style_element)
 
+        self._update_glyphs(element, ranges)
         self._execute_hooks(element)
 
 
@@ -886,52 +899,6 @@ class ElementPlot(BokehPlot, GenericElementPlot):
                    for md in models.values())
 
 
-    @property
-    def current_handles(self):
-        """
-        Returns a list of the plot objects to update.
-        """
-        handles = []
-        if self.static and not self.dynamic:
-            return handles
-
-        for handle in self._update_handles:
-            if (handle == 'source' and self.static_source):
-                continue
-            if handle in self.handles:
-                handles.append(self.handles[handle])
-
-        if self.overlaid:
-            return handles
-
-        plot = self.state
-        handles.append(plot)
-        if bokeh_version >= '0.12':
-            handles.append(plot.title)
-
-        for ax in 'xy':
-            key = '%s_range' % ax
-            if isinstance(self.handles.get(key), FactorRange):
-                handles.append(self.handles[key])
-
-        framewise = self.framewise
-        if self.current_frame:
-            if not self.apply_ranges:
-                rangex, rangey = False, False
-            elif isinstance(self.hmap, DynamicMap):
-                rangex = not self.model_changed(plot.x_range) and framewise
-                rangey = not self.model_changed(plot.y_range) and framewise
-            elif self.framewise:
-                rangex, rangey = True, True
-            else:
-                rangex, rangey = False, False
-            if rangex:
-                handles += [plot.x_range]
-            if rangey:
-                handles += [plot.y_range]
-        return handles
-
-
     @property
     def framewise(self):
         """
@@ -943,6 +910,7 @@ class ElementPlot(BokehPlot, GenericElementPlot):
         current_frames = [el for f in self.traverse(lambda x: x.current_frame)
                           for el in (f.traverse(lambda x: x, [Element])
                                      if f else [])]
+        current_frames = util.unique_iterator(current_frames)
         return any(self.lookup_options(frame, 'norm').options.get('framewise')
                    for frame in current_frames)
 
@@ -958,19 +926,21 @@ class CompositeElementPlot(ElementPlot):
 
     def _init_glyphs(self, plot, element, ranges, source):
         # Get data and initialize data source
-        empty = False
-        if self.batched:
-            current_id = tuple(element.traverse(lambda x: x._plot_id, [Element]))
-            data, mapping = self.get_batched_data(element, ranges, empty)
-        else:
-            data, mapping = self.get_data(element, ranges, empty)
-            current_id = element._plot_id
+        style = self.style[self.cyclic_index]
+        data, mapping, style = self.get_data(element, ranges, style)
 
+        source_cache = {}
+        current_id = element._plot_id
         self.handles['previous_id'] = current_id
         for key in dict(mapping, **data):
-            source = self._init_datasource(data.get(key, {}))
+            ds_data = data.get(key, {})
+            if id(ds_data) in source_cache:
+                source = source_cache[id(ds_data)]
+            else:
+                source = self._init_datasource(ds_data)
+                source_cache[id(ds_data)] = source
             self.handles[key+'_source'] = source
-            properties = self._glyph_properties(plot, element, source, ranges)
+            properties = self._glyph_properties(plot, element, source, ranges, style)
             properties = self._process_properties(key, properties)
             with abbreviated_exception():
                 renderer, glyph = self._init_glyph(plot, mapping.get(key, {}), properties, key)
@@ -986,7 +956,8 @@ class CompositeElementPlot(ElementPlot):
 
 
     def _process_properties(self, key, properties):
-        style_group = self._style_groups[key.split('_')[0]]
+        key = '_'.join(key.split('_')[:-1]) if '_' in key else key
+        style_group = self._style_groups[key]
         group_props = {}
         for k, v in properties.items():
             if k in self.style_opts:
@@ -1000,7 +971,6 @@ class CompositeElementPlot(ElementPlot):
 
     def _update_glyphs(self, element, ranges):
         plot = self.handles['plot']
-        empty = False
 
         # Cache frame object id to skip updating data if unchanged
         previous_id = self.handles.get('previous_id', None)
@@ -1010,16 +980,18 @@ class CompositeElementPlot(ElementPlot):
             current_id = element._plot_id
         self.handles['previous_id'] = current_id
         self.static_source = (self.dynamic and (current_id == previous_id))
-        data, mapping = self.get_data(element, ranges, empty)
+        style = self.style[self.cyclic_index]
+        data, mapping, style = self.get_data(element, ranges, style)
 
-        for key, gdata in data.items():
+        for key in sorted(dict(mapping, **data)):
+            gdata = data.get(key)
             source = self.handles[key+'_source']
-            glyph = self.handles[key+'_glyph']
-            if not self.static_source:
+            glyph = self.handles.get(key+'_glyph')
+            if not self.static_source and gdata is not None:
                 self._update_datasource(source, gdata)
 
             if glyph:
-                properties = self._glyph_properties(plot, element, source, ranges)
+                properties = self._glyph_properties(plot, element, source, ranges, style)
                 properties = self._process_properties(key, properties)
                 renderer = self.handles.get(key+'_glyph_renderer')
                 with abbreviated_exception():
@@ -1093,8 +1065,6 @@ class ColorbarPlot(ElementPlot):
     logz  = param.Boolean(default=False, doc="""
          Whether to apply log scaling to the z-axis.""")
 
-    _update_handles = ['color_mapper', 'source', 'glyph', 'colorbar']
-
     _colorbar_defaults = dict(bar_line_color='black', label_standoff=8,
                               major_tick_line_color='black')
 
@@ -1159,10 +1129,14 @@ class ColorbarPlot(ElementPlot):
         nan_colors = {k: rgba_tuple(v) for k, v in self.clipping_colors.items()}
         colormapper, opts = self._get_cmapper_opts(low, high, factors, nan_colors)
 
-        if 'color_mapper' in self.handles:
+        if 'color_mapper' in self.handles and isinstance(self.handles['color_mapper'], colormapper):
             cmapper = self.handles['color_mapper']
-            cmapper.palette = palette
-            cmapper.update(**opts)
+            if cmapper.palette != palette:
+                cmapper.palette = palette
+            opts = {k: opt for k, opt in opts.items()
+                    if getattr(cmapper, k) != opt}
+            if opts:
+                cmapper.update(**opts)
         else:
             cmapper = colormapper(palette=palette, **opts)
             self.handles['color_mapper'] = cmapper
@@ -1173,17 +1147,19 @@ class ColorbarPlot(ElementPlot):
     def _get_color_data(self, element, ranges, style, name='color', factors=None, colors=None):
         data, mapping = {}, {}
         cdim = element.get_dimension(self.color_index)
-        if cdim:
-            cdata = element.dimension_values(cdim)
-            if factors is None and (isinstance(cdata, list) or cdata.dtype.kind in 'OSU'):
-                factors = list(np.unique(cdata))
-            mapper = self._get_colormapper(cdim, element, ranges, style,
-                                           factors, colors)
-            data[cdim.name] = cdata
-            if factors is not None:
-                mapping['legend'] = {'field': cdim.name}
-            mapping[name] = {'field': cdim.name,
-                             'transform': mapper}
+        if not cdim:
+            return data, mapping
+
+        cdata = element.dimension_values(cdim)
+        if factors is None and (isinstance(cdata, list) or cdata.dtype.kind in 'OSU'):
+            factors = list(np.unique(cdata))
+        mapper = self._get_colormapper(cdim, element, ranges, style,
+                                       factors, colors)
+        data[cdim.name] = cdata
+        if factors is not None:
+            mapping['legend'] = {'field': cdim.name}
+        mapping[name] = {'field': cdim.name,
+                         'transform': mapper}
         return data, mapping
 
 
@@ -1231,13 +1207,15 @@ class LegendPlot(ElementPlot):
         options. The predefined options may be customized in the
         legend_specs class attribute.""")
 
+    legend_offset = param.NumericTuple(default=(0, 0), doc="""
+        If legend is placed outside the axis, this determines the
+        (width, height) offset in pixels from the original position.""")
+
     legend_cols = param.Integer(default=False, doc="""
        Whether to lay out the legend as columns.""")
 
-    legend_specs = {'right': dict(pos='right', loc=(5, -40)),
-                    'left': dict(pos='left', loc=(0, -40)),
-                    'top': dict(pos='above', loc=(120, 5)),
-                    'bottom': dict(pos='below', loc=(60, 0))}
+    legend_specs = {'right': 'right', 'left': 'left', 'top': 'above',
+                    'bottom': 'below'}
 
     def _process_legend(self, plot=None):
         plot = plot or self.handles['plot']
@@ -1249,17 +1227,19 @@ class LegendPlot(ElementPlot):
             categorical = isinstance(cmapper, CategoricalColorMapper)
         else:
             categorical = False
+
         if (not categorical and  not self.overlaid and len(legend.items) == 1) or not self.show_legend:
             legend.items[:] = []
         else:
             plot.legend.orientation = 'horizontal' if self.legend_cols else 'vertical'
             pos = self.legend_position
             if pos in self.legend_specs:
-                opts = self.legend_specs[pos]
                 plot.legend[:] = []
                 legend.plot = None
-                legend.location = opts['loc']
-                plot.add_layout(legend, opts['pos'])
+                legend.location = self.legend_offset
+                if pos in ['top', 'bottom']:
+                    plot.legend.orientation = 'horizontal'
+                plot.add_layout(legend, self.legend_specs[pos])
             else:
                 legend.location = pos
 
@@ -1269,16 +1249,19 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
     tabs = param.Boolean(default=False, doc="""
         Whether to display overlaid plots in separate panes""")
 
-    style_opts = legend_dimensions + line_properties + text_properties
+    style_opts = (legend_dimensions + ['border_'+p for p in line_properties] +
+                  text_properties + ['background_fill_color', 'background_fill_alpha'])
 
-    _update_handles = ['source']
+    multiple_legends = param.Boolean(default=False, doc="""
+        Whether to split the legend for subplots into multiple legends.""")
 
     _propagate_options = ['width', 'height', 'xaxis', 'yaxis', 'labelled',
                           'bgcolor', 'fontsize', 'invert_axes', 'show_frame',
                           'show_grid', 'logx', 'logy', 'xticks', 'toolbar',
                           'yticks', 'xrotation', 'yrotation', 'lod',
                           'border', 'invert_xaxis', 'invert_yaxis', 'sizing_mode',
-                          'title_format']
+                          'title_format', 'legend_position', 'legend_offset',
+                          'legend_cols']
 
     def _process_legend(self):
         plot = self.handles['plot']
@@ -1288,47 +1271,72 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
         options = {}
         properties = self.lookup_options(self.hmap.last, 'style')[self.cyclic_index]
         for k, v in properties.items():
-            if k in line_properties:
-                k = 'border_' + k
-            elif k in text_properties:
+            if k in line_properties and 'line' not in k:
+                ksplit = k.split('_')
+                k = '_'.join(ksplit[:1]+'line'+ksplit[1:])
+            if k in text_properties:
                 k = 'label_' + k
+            if k.startswith('legend_'):
+                k = k[7:]
             options[k] = v
 
-        legend_labels = []
         if not plot.legend:
             return
-        plot.legend[0].update(**options)
-        legend_fontsize = self._fontsize('legend', 'size').get('size',False)
-        if legend_fontsize:
-            plot.legend[0].label_text_font_size = value(legend_fontsize)
 
-        if self.legend_position not in self.legend_specs:
-            plot.legend.location = self.legend_position
-        plot.legend.orientation = 'horizontal' if self.legend_cols else 'vertical'
-        new_legends = []
-        if bokeh_version > '0.12.2':
-            legends = plot.legend[0].items
-            for item in legends:
-                if item.label in legend_labels:
-                    continue
-                legend_labels.append(item.label)
-                new_legends.append(item)
-            plot.legend[0].items[:] = new_legends
+        pos = self.legend_position
+        orientation = 'horizontal' if self.legend_cols else 'vertical'
+        if pos in ['top', 'bottom']:
+            orientation = 'horizontal'
+
+        legend_fontsize = self._fontsize('legend', 'size').get('size',False)
+        legend = plot.legend[0]
+        legend.update(**options)
+        if legend_fontsize:
+            legend.label_text_font_size = value(legend_fontsize)
+
+        if pos in self.legend_specs:
+            pos = self.legend_specs[pos]
         else:
-            legends = plot.legend[0].legends
-            for label, l in legends:
-                if label in legend_labels:
-                    continue
-                legend_labels.append(label)
-                new_legends.append((label, l))
-            plot.legend[0].legends[:] = new_legends
-        if self.legend_position in self.legend_specs:
-            legend = plot.legend[0]
-            plot.legend[:] = []
+            legend.location = pos
+
+        legend.orientation = orientation
+
+        legend_items = []
+        legend_labels = {}
+        for item in legend.items:
+            label = tuple(item.label.items()) if isinstance(item.label, dict) else item.label
+            if not label or (isinstance(item.label, dict) and not item.label.get('value', True)):
+                continue
+            if label in legend_labels:
+                prev_item = legend_labels[label]
+                prev_item.renderers += item.renderers
+            else:
+                legend_labels[label] = item
+                legend_items.append(item)
+        legend.items[:] = legend_items
+
+        if self.multiple_legends:
+            plot.legend.pop(plot.legend.index(legend))
             legend.plot = None
-            leg_opts = self.legend_specs[self.legend_position]
-            legend.location = leg_opts['loc']
-            plot.add_layout(plot.legend[0], leg_opts['pos'])
+            properties = legend.properties_with_values(include_defaults=False)
+            legend_group = []
+            for item in legend.items:
+                if not isinstance(item.label, dict) or 'value'  in item.label:
+                    legend_group.append(item)
+                    continue
+                new_legend = Legend(**dict(properties, items=[item]))
+                new_legend.location = self.legend_offset
+                plot.add_layout(new_legend, pos)
+            if legend_group:
+                new_legend = Legend(**dict(properties, items=legend_group))
+                new_legend.location = self.legend_offset
+                plot.add_layout(new_legend, pos)
+            legend.items[:] = []
+        elif pos in ['above', 'below', 'right', 'left']:
+            plot.legend.pop(plot.legend.index(legend))
+            legend.plot = None
+            legend.location = self.legend_offset
+            plot.add_layout(legend, pos)
 
 
     def _init_tools(self, element, callbacks=[]):
@@ -1348,10 +1356,11 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
                     else:
                         tool_type = type(tool)
                     if isinstance(tool, HoverTool):
-                        if tuple(tool.tooltips) in hover_tools:
+                        tooltips = tuple(tool.tooltips) if tool.tooltips else ()
+                        if tooltips in hover_tools:
                             continue
                         else:
-                            hover_tools[tuple(tool.tooltips)] = tool
+                            hover_tools[tooltips] = tool
                     elif tool_type in tool_types:
                         continue
                     else:
@@ -1371,9 +1380,12 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
             hover = subplot.handles['hover']
             # Datetime formatter may have been applied, remove _dt_strings
             # to match on the hover tooltips, then merge tool renderers
-            tooltips = [(name, spec.replace('_dt_strings', ''))
-                        for name, spec in hover.tooltips]
-            tool = self.handles['hover_tools'].get(tuple(tooltips))
+            if hover.tooltips:
+                tooltips = tuple((name, spec.replace('_dt_strings', ''))
+                                  for name, spec in hover.tooltips)
+            else:
+                tooltips = ()
+            tool = self.handles['hover_tools'].get(tooltips)
             if tool:
                 renderers = tool.renderers+hover.renderers
                 tool.renderers = list(util.unique_iterator(renderers))
@@ -1435,6 +1447,12 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
             self._process_legend()
         self.drawn = True
 
+        if 'plot' in self.handles and not self.tabs:
+            plot = self.handles['plot']
+            self.handles['xaxis'] = plot.xaxis[0]
+            self.handles['yaxis'] = plot.yaxis[0]
+            self.handles['x_range'] = plot.x_range
+            self.handles['y_range'] = plot.y_range
         for cb in self.callbacks:
             cb.initialize()
 
@@ -1443,7 +1461,7 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
         return self.handles['plot']
 
 
-    def update_frame(self, key, ranges=None, element=None, empty=False):
+    def update_frame(self, key, ranges=None, element=None):
         """
         Update the internal state of the Plot to represent the given
         key tuple (where integers represent frames). Returns this
@@ -1462,24 +1480,25 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
         else:
             range_obj = self.hmap
 
-        all_empty = empty
         if element is not None:
             ranges = self.compute_ranges(range_obj, key, ranges)
+
+        if element and not self.overlaid and not self.tabs and not self.batched:
+            self._update_ranges(element, ranges)
+            
         for k, subplot in self.subplots.items():
-            empty, el = False, None
+            el = None
             # If in Dynamic mode propagate elements to subplots
             if isinstance(self.hmap, DynamicMap) and element:
                 # In batched mode NdOverlay is passed to subplot directly
                 if self.batched:
                     el = element
-                    empty = False
                 # If not batched get the Element matching the subplot
                 elif element is not None:
                     idx = dynamic_update(self, subplot, k, element, items)
-                    empty = idx is None
-                    if not empty:
+                    if idx is not None:
                         _, el = items.pop(idx)
-            subplot.update_frame(key, ranges, element=el, empty=(empty or all_empty))
+            subplot.update_frame(key, ranges, element=el)
 
         if not self.batched and isinstance(self.hmap, DynamicMap) and items:
             self.warning("Some Elements returned by the dynamic callback "
@@ -1487,7 +1506,6 @@ class OverlayPlot(GenericOverlayPlot, LegendPlot):
                          "rendered.")
 
         if element and not self.overlaid and not self.tabs and not self.batched:
-            self._update_ranges(element, ranges)
             self._update_plot(key, self.handles['plot'], element)
 
         self._execute_hooks(element)
