@@ -1,27 +1,23 @@
+import json
 from itertools import groupby
-import numpy as np
 
+import numpy as np
 import param
 
 from bokeh.models import (ColumnDataSource, Column, Row, Div)
 from bokeh.models.widgets import Panel, Tabs
 
-from ...core import (OrderedDict, CompositeOverlay, Store, GridMatrix,
-                     AdjointLayout, NdLayout, Empty, GridSpace, HoloMap, Element)
-from ...core.options import Compositor
+from ...core import (OrderedDict, Store, GridMatrix, AdjointLayout,
+                     NdLayout, Empty, GridSpace, HoloMap, Element)
 from ...core.util import basestring, wrap_tuple, unique_iterator
 from ...element import Histogram
 from ..plot import (DimensionedPlot, GenericCompositePlot, GenericLayoutPlot,
                     GenericElementPlot)
 from ..util import attach_streams
-from .util import (bokeh_version, layout_padding, pad_plots,
-                   filter_toolboxes, make_axis, update_shared_sources,
-                   empty_plot)
+from .util import (layout_padding, pad_plots, filter_toolboxes, make_axis,
+                   update_shared_sources, empty_plot)
 
-if bokeh_version >= '0.12':
-    from bokeh.layouts import gridplot
-else:
-    from bokeh.models import GridPlot as BokehGridPlot
+from bokeh.layouts import gridplot
 from bokeh.plotting.helpers import _known_tools as known_tools
 
 TOOLS = {name: tool if isinstance(tool, basestring) else type(tool())
@@ -93,7 +89,7 @@ class BokehPlot(DimensionedPlot):
         self.root = None
 
 
-    def get_data(self, element, ranges=None, empty=False):
+    def get_data(self, element, ranges, style):
         """
         Returns the data from an element in the appropriate format for
         initializing or updating a ColumnDataSource and a dictionary
@@ -111,8 +107,16 @@ class BokehPlot(DimensionedPlot):
             return
         if self.comm is None:
             raise Exception('Renderer does not have a comm.')
-        diff = self.renderer.diff(self)
-        self.comm.send(diff)
+
+        msg = self.renderer.diff(self, binary=True)
+        if msg is None:
+            return
+        self.comm.send(msg.header_json)
+        self.comm.send(msg.metadata_json)
+        self.comm.send(msg.content_json)
+        for header, payload in msg.buffers:
+            self.comm.send(json.dumps(header))
+            self.comm.send(buffers=[payload])
 
 
     def set_root(self, root):
@@ -134,7 +138,14 @@ class BokehPlot(DimensionedPlot):
         """
         Update datasource with data for a new frame.
         """
-        source.data.update(data)
+        if (self.streaming and self.streaming[0].data is self.current_frame.data
+            and self._stream_data):
+            stream = self.streaming[0]
+            if stream._triggering:
+                data = {k: v[-stream._chunk_length:] for k, v in data.items()}
+                source.stream(data, stream.length)
+        else:
+            source.data.update(data)
 
     @property
     def state(self):
@@ -322,9 +333,6 @@ class GridPlot(CompositePlot, GenericCompositePlot):
 
 
     def _create_subplots(self, layout, ranges):
-        layout = layout.map(Compositor.collapse_element, [CompositeOverlay],
-                            clone=False)
-
         subplots = OrderedDict()
         frame_ranges = self.compute_ranges(layout, None, ranges)
         frame_ranges = OrderedDict([(key, self.compute_ranges(layout, key, frame_ranges))
@@ -412,11 +420,7 @@ class GridPlot(CompositePlot, GenericCompositePlot):
             else:
                 passed_plots.append(None)
 
-        if bokeh_version < '0.12':
-            plot = BokehGridPlot(children=plots[::-1])
-        else:
-            plot = gridplot(plots[::-1])
-
+        plot = gridplot(plots[::-1])
         plot = self._make_axes(plot)
 
         title = self._get_title(self.keys[-1])
@@ -647,7 +651,6 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
         plots = [[] for _ in range(self.rows)]
         tab_titles = {}
         insert_rows, insert_cols = [], []
-        adjoined = False
         for r, c in self.coords:
             subplot = self.subplots.get((r, c), None)
             if subplot is not None:
@@ -658,7 +661,6 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
                 # number of adjoined plots
                 offset = sum(r >= ir for ir in insert_rows)
                 if len(subplots) > 2:
-                    adjoined = True
                     # Add pad column in this position
                     insert_cols.append(c)
                     if r not in insert_rows:
@@ -673,7 +675,6 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
                     # Add top marginal
                     plots[r+offset-1] += [subplots.pop(-1), None]
                 elif len(subplots) > 1:
-                    adjoined = True
                     # Add pad column in this position
                     insert_cols.append(c)
                     # Pad previous rows
@@ -711,16 +712,10 @@ class LayoutPlot(CompositePlot, GenericLayoutPlot):
                       for c, child in enumerate(row)
                       if child is not None]
             layout_plot = Tabs(tabs=panels)
-        elif bokeh_version >= '0.12':
+        else:
             plots = filter_toolboxes(plots)
             plots, width = pad_plots(plots)
             layout_plot = gridplot(children=plots, width=width, **kwargs)
-        elif len(plots) == 1 and not adjoined:
-            layout_plot = Column(children=[Row(children=plots[0])], **kwargs)
-        elif len(plots[0]) == 1:
-            layout_plot = Column(children=[p[0] for p in plots], **kwargs)
-        else:
-            layout_plot = BokehGridPlot(children=plots, **kwargs)
 
         title = self._get_title(self.keys[-1])
         if title:
