@@ -3,7 +3,7 @@ import param
 
 from bokeh.models import HoverTool
 from ...core.util import cartesian_product, is_nan, dimension_sanitizer
-from ...element import Image, Raster
+from ...element import Raster
 from ..renderer import SkipRendering
 from .element import ElementPlot, ColorbarPlot, line_properties, fill_properties
 
@@ -21,45 +21,40 @@ class RasterPlot(ColorbarPlot):
         if self.hmap.type == Raster:
             self.invert_yaxis = not self.invert_yaxis
 
+    def get_data(self, element, ranges, style):
+        mapping = dict(image='image', x='x', y='y', dw='dw', dh='dh')
+        val_dim = [d for d in element.vdims][0]
+        style['color_mapper'] = self._get_colormapper(val_dim, element, ranges, style)
+
+        if self.static_source:
+            return {}, mapping, style
 
-    def get_data(self, element, ranges=None, empty=False):
         img = element.dimension_values(2, flat=False)
         if img.dtype.kind == 'b':
             img = img.astype(np.int8)
 
-        if isinstance(element, Image):
-            l, b, r, t = element.bounds.lbrt()
-        else:
-            img = img.T[::-1] if self.invert_yaxis else img.T
+        if type(element) is Raster:
             l, b, r, t = element.extents
+            if self.invert_axes:
+                l, b, r, t = b, l, t, r
+            else:
+                img = img.T
+        else:
+            l, b, r, t = element.bounds.lbrt()
+            if self.invert_axes:
+                img = img.T
+                l, b, r, t = b, l, t, r
 
-        # Ensure axis inversions are handled correctly
         if self.invert_xaxis:
             l, r = r, l
             img = img[:, ::-1]
         if self.invert_yaxis:
+            img = img[::-1]
             b, t = t, b
-            if type(element) is not Raster:
-                img = img[::-1]
         dh, dw = t-b, r-l
 
-        mapping = dict(image='image', x='x', y='y', dw='dw', dh='dh')
-        if empty:
-            data = dict(image=[], x=[], y=[], dw=[], dh=[])
-        else:
-            data = dict(image=[img], x=[l],
-                        y=[b], dw=[dw], dh=[dh])
-        return (data, mapping)
-
-
-    def _glyph_properties(self, plot, element, source, ranges):
-        properties = super(RasterPlot, self)._glyph_properties(plot, element,
-                                                               source, ranges)
-        properties = {k: v for k, v in properties.items()}
-        val_dim = [d for d in element.vdims][0]
-        properties['color_mapper'] = self._get_colormapper(val_dim, element, ranges,
-                                                           properties)
-        return properties
+        data = dict(image=[img], x=[l], y=[b], dw=[dw], dh=[dh])
+        return (data, mapping, style)
 
 
 
@@ -68,7 +63,11 @@ class RGBPlot(RasterPlot):
     style_opts = []
     _plot_methods = dict(single='image_rgba')
 
-    def get_data(self, element, ranges=None, empty=False):
+    def get_data(self, element, ranges, style):
+        mapping = dict(image='image', x='x', y='y', dw='dw', dh='dh')
+        if self.static_source:
+            return {}, mapping, style
+
         img = np.dstack([element.dimension_values(d, flat=False)
                          for d in element.vdims])
         if img.ndim == 3:
@@ -85,30 +84,28 @@ class RGBPlot(RasterPlot):
 
         # Ensure axis inversions are handled correctly
         l, b, r, t = element.bounds.lbrt()
+        if self.invert_axes:
+            img = img.T
+            l, b, r, t = b, l, t, r
         if self.invert_xaxis:
             l, r = r, l
             img = img[:, ::-1]
         if self.invert_yaxis:
-            b, t = t, b
             img = img[::-1]
+            b, t = t, b
         dh, dw = t-b, r-l
 
-        mapping = dict(image='image', x='x', y='y', dw='dw', dh='dh')
-        if empty:
-            data = dict(image=[], x=[], y=[], dw=[], dh=[])
-        else:
-            data = dict(image=[img], x=[l],
-                        y=[b], dw=[dw], dh=[dh])
-        return (data, mapping)
+        data = dict(image=[img], x=[l], y=[b], dw=[dw], dh=[dh])
+        return (data, mapping, style)
 
-    def _glyph_properties(self, plot, element, source, ranges):
+    def _glyph_properties(self, plot, element, source, ranges, style):
         return ElementPlot._glyph_properties(self, plot, element,
-                                             source, ranges)
+                                             source, ranges, style)
 
 class HSVPlot(RGBPlot):
 
-    def get_data(self, element, ranges=None, empty=False):
-        return super(HSVPlot, self).get_data(element.rgb, ranges, empty)
+    def get_data(self, element, ranges, style):
+        return super(HSVPlot, self).get_data(element.rgb, ranges, style)
 
 
 class HeatMapPlot(ColorbarPlot):
@@ -127,35 +124,41 @@ class HeatMapPlot(ColorbarPlot):
     _plot_methods = dict(single='rect')
     style_opts = ['cmap', 'color'] + line_properties + fill_properties
 
-    _update_handles = ['color_mapper', 'source', 'glyph', 'colorbar']
     _categorical = True
 
     def _get_factors(self, element):
         return super(HeatMapPlot, self)._get_factors(element.gridded)
 
-    def get_data(self, element, ranges=None, empty=False):
+    def get_data(self, element, ranges, style):
         x, y, z = [dimension_sanitizer(d) for d in element.dimensions(label=True)[:3]]
-        aggregate = element.gridded
-        style = self.style[self.cyclic_index]
+        if self.invert_axes: x, y = y, x
         cmapper = self._get_colormapper(element.vdims[0], element, ranges, style)
-        if empty:
-            data = {x: [], y: [], z: []}
-        else:
-            xdim, ydim = aggregate.dimensions()[:2]
-            xvals, yvals, zvals = (aggregate.dimension_values(i) for i in range(3))
-            if xvals.dtype.kind not in 'SU':
-                xvals = [xdim.pprint_value(xv) for xv in xvals]
-            if yvals.dtype.kind not in 'SU':
-                yvals = [ydim.pprint_value(yv) for yv in yvals]
-            data = {x: xvals, y: yvals, 'zvalues': zvals}
+        if self.static_source:
+            return {}, {'x': x, 'y': y, 'fill_color': {'field': 'zvalues', 'transform': cmapper}}, style
 
-        if any(isinstance(t, HoverTool) for t in self.state.tools):
+        aggregate = element.gridded
+        xdim, ydim = aggregate.dimensions()[:2]
+        xvals, yvals = (aggregate.dimension_values(x),
+                        aggregate.dimension_values(y))
+        zvals = aggregate.dimension_values(2, flat=False)
+        if self.invert_axes:
+            xdim, ydim = ydim, xdim
+            zvals = zvals.T.flatten()
+        else:
+            zvals = zvals.T.flatten()
+        if xvals.dtype.kind not in 'SU':
+            xvals = [xdim.pprint_value(xv) for xv in xvals]
+        if yvals.dtype.kind not in 'SU':
+            yvals = [ydim.pprint_value(yv) for yv in yvals]
+        data = {x: xvals, y: yvals, 'zvalues': zvals}
+
+        if any(isinstance(t, HoverTool) for t in self.state.tools) and not self.static_source:
             for vdim in element.vdims:
                 sanitized = dimension_sanitizer(vdim.name)
                 data[sanitized] = ['-' if is_nan(v) else vdim.pprint_value(v)
                                    for v in aggregate.dimension_values(vdim)]
         return (data, {'x': x, 'y': y, 'fill_color': {'field': 'zvalues', 'transform': cmapper},
-                       'height': 1, 'width': 1})
+                       'height': 1, 'width': 1}, style)
 
 
 class QuadMeshPlot(ColorbarPlot):
@@ -166,24 +169,28 @@ class QuadMeshPlot(ColorbarPlot):
     _plot_methods = dict(single='rect')
     style_opts = ['cmap', 'color'] + line_properties + fill_properties
 
-    def get_data(self, element, ranges=None, empty=False):
+    def get_data(self, element, ranges, style):
         x, y, z = element.dimensions(label=True)
-        style = self.style[self.cyclic_index]
+        if self.invert_axes: x, y = y, x
         cmapper = self._get_colormapper(element.vdims[0], element, ranges, style)
-        if empty:
-            data = {x: [], y: [], z: [], 'height': [], 'width': []}
-        else:
-            if len(set(v.shape for v in element.data)) == 1:
-                raise SkipRendering("Bokeh QuadMeshPlot only supports rectangular meshes")
-            zvals = element.data[2].T.flatten()
-            xvals = element.dimension_values(0, False)
-            yvals = element.dimension_values(1, False)
-            widths = np.diff(element.data[0])
-            heights = np.diff(element.data[1])
-            xs, ys = cartesian_product([xvals, yvals], copy=True)
-            ws, hs = cartesian_product([widths, heights], copy=True)
-            data = {x: xs, y: ys, z: zvals, 'widths': ws, 'heights': hs}
+        if self.static_source:
+            return {}, {'x': x, 'y': y, 'fill_color': {'field': z, 'transform': cmapper}}, style
 
+        if len(set(v.shape for v in element.data)) == 1:
+            raise SkipRendering("Bokeh QuadMeshPlot only supports rectangular meshes")
+        zdata = element.data[2]
+        xvals = element.dimension_values(0, False)
+        yvals = element.dimension_values(1, False)
+        widths = np.diff(element.data[0])
+        heights = np.diff(element.data[1])
+        if self.invert_axes:
+            zvals = zdata.flatten()
+            xvals, yvals, widths, heights = yvals, xvals, heights, widths
+        else:
+            zvals = zdata.T.flatten()
+        xs, ys = cartesian_product([xvals, yvals], copy=True)
+        ws, hs = cartesian_product([widths, heights], copy=True)
+        data = {x: xs, y: ys, z: zvals, 'widths': ws, 'heights': hs}
         return (data, {'x': x, 'y': y,
                        'fill_color': {'field': z, 'transform': cmapper},
-                       'height': 'heights', 'width': 'widths'})
+                       'height': 'heights', 'width': 'widths'}, style)
