@@ -130,23 +130,26 @@ def compute_overlayable_zorders(obj, path=[]):
     # Process the inputs of the DynamicMap callback
     dmap_inputs = obj.callback.inputs if obj.callback.link_inputs else []
     for z, inp in enumerate(dmap_inputs):
+        no_zorder_increment = False
         if any(not (isoverlay_fn(p) or p.last is None) for p in path) and isoverlay_fn(inp):
-            # Skips branches of graph that collapse Overlay layers
-            # to avoid adding layers that have been reduced or removed
-            continue
+            # If overlay has been collapsed do not increment zorder
+            no_zorder_increment = True
 
         input_depth = overlay_depth(inp)
         if depth is not None and input_depth is not None and depth < input_depth:
             # Skips branch of graph where the number of elements in an
-            # overlay has been reduced
-            continue
+            # overlay has been reduced but still contains more than one layer
+            if depth > 1:
+                continue
+            else:
+                no_zorder_increment = True
 
         # Recurse into DynamicMap.callback.inputs and update zorder_map
         z = z if isdynoverlay else 0
         deep_zorders = compute_overlayable_zorders(inp, path=path)
         offset = max(zorder_map.keys())
         for dz, objs in deep_zorders.items():
-            global_z = offset+dz+z
+            global_z = offset+z if no_zorder_increment else offset+dz+z
             zorder_map[global_z] = list(unique_iterator(zorder_map[global_z]+objs))
 
     # If object branches but does not declare inputs (e.g. user defined
@@ -249,19 +252,19 @@ def get_sideplot_ranges(plot, element, main, ranges):
     dictionary of ranges.
     """
     key = plot.current_key
-    dims = element.dimensions(label=True)
-    dim = dims[1] if dims[1] != 'Frequency' else dims[0]
+    dims = element.dimensions()
+    dim = dims[0] if 'frequency' in dims[1].name else dims[1]
     range_item = main
     if isinstance(main, HoloMap):
         if issubclass(main.type, CompositeOverlay):
             range_item = [hm for hm in main.split_overlays()[1]
-                          if dim in hm.dimensions('all', label=True)][0]
+                          if dim in hm.dimensions('all')][0]
     else:
         range_item = HoloMap({0: main}, kdims=['Frame'])
         ranges = match_spec(range_item.last, ranges)
 
-    if dim in ranges:
-        main_range = ranges[dim]
+    if dim.name in ranges:
+        main_range = ranges[dim.name]
     else:
         framewise = plot.lookup_options(range_item.last, 'norm').options.get('framewise')
         if framewise and range_item.get(key, False):
@@ -274,12 +277,14 @@ def get_sideplot_ranges(plot, element, main, ranges):
         range_item = range_item.last
     if isinstance(range_item, CompositeOverlay):
         range_item = [ov for ov in range_item
-                      if dim in ov.dimensions('all', label=True)][0]
+                      if dim in ov.dimensions('all')][0]
     return range_item, main_range, dim
 
 
 def within_range(range1, range2):
     """Checks whether range1 is within the range specified by range2."""
+    range1 = [r if np.isfinite(r) else None for r in range1]
+    range2 = [r if np.isfinite(r) else None for r in range2]
     return ((range1[0] is None or range2[0] is None or range1[0] >= range2[0]) and
             (range1[1] is None or range2[1] is None or range1[1] <= range2[1]))
 
@@ -438,11 +443,14 @@ def get_min_distance(element):
     Gets the minimum sampling distance of the x- and y-coordinates
     in a grid.
     """
-    xys = element.array([0, 1]).view(dtype=np.complex128)
+    xys = element.array([0, 1]).astype('float64').view(dtype=np.complex128)
     m, n = np.meshgrid(xys, xys)
     distances = np.abs(m-n)
     np.fill_diagonal(distances, np.inf)
-    return distances[distances>0].min()
+    distances = distances[distances>0]
+    if len(distances):
+        return distances.min()
+    return 0
 
 
 def rgb2hex(rgb):
