@@ -4,6 +4,8 @@ generate and respond to events, originating either in Python on the
 server-side or in Javascript in the Jupyter notebook (client-side).
 """
 
+import uuid
+
 import param
 import numpy as np
 from numbers import Number
@@ -191,7 +193,7 @@ class Stream(param.Parameterized):
     @property
     def subscribers(self):
         " Property returning the subscriber list"
-        return [s for p, s in sorted(self._subscribers)]
+        return [s for p, s in sorted(self._subscribers, key=lambda x: x[0])]
 
 
     def clear(self, policy='all'):
@@ -296,6 +298,15 @@ class Stream(param.Parameterized):
         return {self._rename.get(k,k):v for (k,v) in filtered.items()
                 if (self._rename.get(k,True) is not None)}
 
+    @property
+    def hashkey(self):
+        """
+        The object the memoization hash is computed from. By default
+        returns the stream contents but can be overridden to provide
+        a custom hash key.
+        """
+        return self.contents
+
 
     def _set_stream_parameters(self, **kwargs):
         """
@@ -352,6 +363,181 @@ class Counter(Stream):
         return {'counter': self.counter + 1}
 
 
+class Pipe(Stream):
+    """
+    A Stream used to pipe arbitrary data to a callback.
+    Unlike other streams memoization can be disabled for a
+    Pipe stream (and is disabled by default).
+    """
+
+    data = param.Parameter(default=None, constant=True, doc="""
+        Arbitrary data being streamed to a DynamicMap callback.""")
+
+    def __init__(self, data=None, memoize=False, **params):
+        super(Pipe, self).__init__(data=data, **params)
+        self._memoize = memoize
+
+    def send(self, data):
+        """
+        A convenience method to send an event with data without
+        supplying a keyword.
+        """
+        self.event(data=data)
+
+    @property
+    def hashkey(self):
+        if self._memoize:
+            return self.contents
+        return {'hash': uuid.uuid4().hex}
+
+
+class Buffer(Pipe):
+    """
+    Buffer allows streaming and accumulating incoming chunks of rows
+    from tabular datasets. The data may be in the form of a pandas
+    DataFrame, 2D arrays of rows and columns or dictionaries of column
+    arrays. Buffer will accumulate the last N rows, where N is defined
+    by the specified ``length``. The accumulated data is then made
+    available via the ``data`` parameter.
+
+    A Buffer may also be instantiated with a streamz.StreamingDataFrame
+    or a streamz.StreamingSeries, it will automatically subscribe to
+    events emitted by a streamz object.
+
+    When streaming a DataFrame will reset the DataFrame index by
+    default making it available to HoloViews elements as dimensions,
+    this may be disabled by setting index=False.
+    """
+
+    def __init__(self, data, length=1000, index=True, **params):
+        if (util.pd and isinstance(data, util.pd.DataFrame)):
+            example = data
+        elif isinstance(data, np.ndarray):
+            if data.ndim != 2:
+                raise ValueError("Only 2D array data may be streamed by Buffer.")
+            example = data
+        elif isinstance(data, dict):
+            if not all(isinstance(v, np.ndarray) for v in data.values()):
+                raise ValueError("Data in dictionary must be of array types.")
+            elif len(set(len(v) for v in data.values())) > 1:
+                raise ValueError("Columns in dictionary must all be the same length.")
+            example = data
+        else:
+            try:
+                from streamz.dataframe import StreamingDataFrame, StreamingSeries
+                loaded = True
+            except ImportError:
+                loaded = False
+            if not loaded or not isinstance(data, (StreamingDataFrame, StreamingSeries)):
+                raise ValueError("Buffer must be initialized with pandas DataFrame, "
+                                 "streamz.StreamingDataFrame or streamz.StreamingSeries.")
+            elif isinstance(data, StreamingSeries):
+                data = data.to_frame()
+            example = data.example
+            data.stream.sink(self.send)
+            self.sdf = data
+
+        if index and (util.pd and isinstance(example, util.pd.DataFrame)):
+            example = example.reset_index()
+        params['data'] = example
+        super(Buffer, self).__init__(**params)
+        self.length = length
+        self._chunk_length = 0
+        self._count = 0
+        self._index = index
+
+
+    def verify(self, x):
+        """ Verify consistency of dataframes that pass through this stream """
+        if type(x) != type(self.data):
+            raise TypeError("Input expected to be of type %s, got %s." %
+                            (type(self.data).__name__, type(x).__name__))
+        elif isinstance(x, np.ndarray):
+            if x.ndim != 2:
+                raise ValueError('Streamed array data must be two-dimensional')
+            elif x.shape[1] != self.data.shape[1]:
+                raise ValueError("Streamed array data expeced to have %d columns, "
+                                 "got %d." % (self.data.shape[1], x.shape[1]))
+        elif util.pd and isinstance(x, util.pd.DataFrame) and list(x.columns) != list(self.data.columns):
+            raise IndexError("Input expected to have columns %s, got %s" %
+                             (list(self.data.columns), list(x.columns)))
+        elif isinstance(x, dict):
+            if any(c not in x for c in self.data):
+                raise IndexError("Input expected to have columns %s, got %s" %
+                                 (sorted(self.data.keys()), sorted(x.keys())))
+            elif len(set(len(v) for v in x.values())) > 1:
+                raise ValueError("Input columns expected to have the "
+                                 "same number of rows.")
+
+
+    def clear(self):
+        "Clears the data in the stream"
+        if isinstance(self.data, np.ndarray):
+            data = self.data[:, :0]
+        elif util.pd and isinstance(self.data, util.pd.DataFrame):
+            data = self.data.iloc[:0]
+        elif isinstance(self.data, dict):
+            data = {k: v[:0] for k, v in self.data.items()}
+        with util.disable_constant(self):
+            self.data = data
+        self.send(data)
+
+
+    def _concat(self, data):
+        """
+        Concatenate and slice the accepted data types to the defined
+        length.
+        """
+        if isinstance(data, np.ndarray):
+            data_length = len(data)
+            if data_length < self.length:
+                prev_chunk = self.data[-(self.length-data_length):]
+                data = np.concatenate([prev_chunk, data])
+            elif data_length > self.length:
+                data = data[-self.length:]
+        elif util.pd and isinstance(data, util.pd.DataFrame):
+            data_length = len(data)
+            if data_length < self.length:
+                prev_chunk = self.data.iloc[-(self.length-data_length):]
+                data = util.pd.concat([prev_chunk, data])
+            elif data_length > self.length:
+                data = data.iloc[-self.length:]
+        elif isinstance(data, dict) and data:
+            data_length = len(list(data.values())[0])
+            new_data = {}
+            for k, v in data.items():
+                if data_length < self.length:
+                    prev_chunk = self.data[k][-(self.length-data_length):]
+                    new_data[k] = np.concatenate([prev_chunk, v])
+                elif data_length > self.length:
+                    new_data[k] = v[-self.length:]
+                else:
+                    new_data[k] = v
+            data = new_data
+        self._chunk_length = data_length
+        return data
+
+
+    def update(self, **kwargs):
+        """
+        Overrides update to concatenate streamed data up to defined length.
+        """
+        data = kwargs.get('data')
+        if data is not None:
+            if util.pd and isinstance(data, util.pd.DataFrame) and self._index:
+                data = data.reset_index()
+            self.verify(data)
+            kwargs['data'] = self._concat(data)
+            self._count += 1
+        super(Buffer, self).update(**kwargs)
+
+
+    @property
+    def hashkey(self):
+        return {'hash': self._count}
+
+
+
 class LinkedStream(Stream):
     """
     A LinkedStream indicates is automatically linked to plot interactions
@@ -404,11 +590,11 @@ class PointerXY(LinkedStream):
     the plot bounds, the position values are set to None.
     """
 
-    x = param.ClassSelector(class_=(Number, util.basestring), default=None,
+    x = param.ClassSelector(class_=(Number, util.basestring, tuple), default=None,
                             constant=True, doc="""
            Pointer position along the x-axis in data coordinates""")
 
-    y = param.ClassSelector(class_=(Number, util.basestring), default=None,
+    y = param.ClassSelector(class_=(Number, util.basestring, tuple), default=None,
                             constant=True, doc="""
            Pointer position along the y-axis in data coordinates""")
 
@@ -477,10 +663,10 @@ class RangeXY(LinkedStream):
     Axis ranges along x- and y-axis in data coordinates.
     """
 
-    x_range = param.NumericTuple(default=None, length=2, constant=True, doc="""
+    x_range = param.Tuple(default=None, length=2, constant=True, doc="""
       Range of the x-axis of a plot in data coordinates""")
 
-    y_range = param.NumericTuple(default=None, length=2, constant=True, doc="""
+    y_range = param.Tuple(default=None, length=2, constant=True, doc="""
       Range of the y-axis of a plot in data coordinates""")
 
 
@@ -489,7 +675,7 @@ class RangeX(LinkedStream):
     Axis range along x-axis in data coordinates.
     """
 
-    x_range = param.NumericTuple(default=None, length=2, constant=True, doc="""
+    x_range = param.Tuple(default=None, length=2, constant=True, doc="""
       Range of the x-axis of a plot in data coordinates""")
 
 
@@ -498,7 +684,7 @@ class RangeY(LinkedStream):
     Axis range along y-axis in data coordinates.
     """
 
-    y_range = param.NumericTuple(default=None, length=2, constant=True, doc="""
+    y_range = param.Tuple(default=None, length=2, constant=True, doc="""
       Range of the y-axis of a plot in data coordinates""")
 
 
@@ -508,8 +694,8 @@ class BoundsXY(LinkedStream):
     tuple of the left, bottom, right and top coordinates.
     """
 
-    bounds = param.NumericTuple(default=None, constant=True, length=4,
-                                allow_None=True, doc="""
+    bounds = param.Tuple(default=None, constant=True, length=4,
+                         allow_None=True, doc="""
         Bounds defined as (left, bottom, right, top) tuple.""")
 
 
@@ -519,14 +705,15 @@ class Bounds(BoundsXY):
         self.warning('Bounds is deprecated use BoundsXY instead.')
         super(Bounds, self).__init__(*args, **kwargs)
 
+
 class BoundsX(LinkedStream):
     """
     A stream representing the bounds of a box selection as an
     tuple of the left and right coordinates.
     """
 
-    boundsx = param.NumericTuple(default=None, constant=True, length=2,
-                                 allow_None=True, doc="""
+    boundsx = param.Tuple(default=None, constant=True, length=2,
+                          allow_None=True, doc="""
         Bounds defined as (left, right) tuple.""")
 
 
@@ -536,8 +723,8 @@ class BoundsY(LinkedStream):
     tuple of the bottom and top coordinates.
     """
 
-    boundsy = param.NumericTuple(default=None, constant=True, length=2,
-                                 allow_None=True, doc="""
+    boundsy = param.Tuple(default=None, constant=True, length=2,
+                          allow_None=True, doc="""
         Bounds defined as (bottom, top) tuple.""")
 
 
@@ -550,6 +737,18 @@ class Selection1D(LinkedStream):
         Indices into a 1D datastructure.""")
 
 
+class PlotReset(LinkedStream):
+    """
+    A stream signalling when a plot reset event has been triggered.
+    """
+
+    reset = param.Boolean(default=False, constant=True, doc="""
+        Whether a reset event is being signalled.""")
+
+    def __init__(self, *args, **params):
+        super(PlotReset, self).__init__(self, *args, **dict(params, transient=True))
+
+
 class ParamValues(Stream):
     """
     A Stream based on the parameter values of some other parameterized
@@ -612,3 +811,4 @@ class PositionXY(PointerXY):
     def __init__(self, **params):
         self.warning('PositionXY stream deprecated: use PointerXY instead')
         super(PositionXY, self).__init__(**params)
+
